@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_master - remote control master for Nokia N900 phone #
-# $Id: smscon_master.py 123 2011-12-29 21:53:26Z yablacky $
+# $Id: smscon_master.py 139 2012-01-08 22:37:54Z yablacky $
 ##############################################################
 
 import os
@@ -117,7 +117,7 @@ def ShowOptions(Level = 1):
         print '   -unboot            : Disable %s to start at phone boot' % DaemonFile
         print '   -add imsi          : Add IMSI code of current SIM to %s file' % CodeDisplayName
         print '   -remove imsi       : Remove IMSI code of current SIM from %s file' % CodeDisplayName
-        print '   -pw password       : Password to enable non-anomymized output. MASTERNUMBER is password.'
+        print '   -pw password       : Password to enable non-anonymized output. MASTERNUMBER is password.'
         print '   -version           : Show version number'    
         print '   -help              : This help menu'
         print '   -help2             : More help (special options, rarely used).'
@@ -200,7 +200,7 @@ def DoCommandLine():
                 ok = ShowFile(LOGGER.LogFilename, ReplacementRules = [
                     ('\\+?[0-9]{8,}', '+00123456789'),                          # replace potential phone numbers
                     ('([a-zA-Z][a-zA-Z0-9_\\-]*\\.)+[a-zA-Z0-9_\\-]*[a-zA-Z]', 'example.com'),  # replace potential host names
-                    ('(?:[0-9]{1,3}\\.){1,3}([0-9]{1,3})', '192.168.128.\\1'),  # replace potential IPv4
+                    ('(?:[0-9]{1,3}\\.){2,3}([0-9]{1,3})', '192.168.128.\\1'),  # replace potential IPv4 (but no floats)
                 ])    
         elif Mode == '-imsi':       ok = ShowIMSICodes()
         elif Mode == '-sms':        ok = ShowSmsCommands(ourConf)
