@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 121 2011-12-29 20:14:19Z yablacky $
+# $Id: smscon_service.py 139 2012-01-08 22:37:54Z yablacky $
 ##############################################################
 
 # standard modules
@@ -46,6 +46,7 @@ GpsCoordinateList           = []
 GpsMode                     = False                             # False if not active or mode-name if GPS device is enabled
 GpsIsTest                   = False                             # true if GPS device is enabled from test mode command
 GpsStartTime                = 0
+GpsControl                  = None
 
 ConnectionType              = None 
 ConnectionIAP               = None
@@ -73,6 +74,17 @@ def GetTimeString(*fromTime):
     """
     return time.strftime(smscnf.LOG_TIMEFORMAT, *fromTime)
 
+def SaveReadFileLine(Filename, DefaultValue = 'unknown',  ErrorMessage = None):
+    """
+    Read one line of text with EOL char stripped of. Exception save.
+    @return string.
+    """
+    try:
+        return open(Filename).read().strip()
+    except Exception, e:
+        LOGGER.error(IF(ErrorMessage,  ErrorMessage + ': %s', '%s') % e)
+    return DefaultValue
+
 #############################################
 
 def RevealThat(Subject, Text, SendMethod = None, Attachment = None, Attachhint = None):
@@ -563,7 +575,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
         PerformPoweroff()            
 
     elif Command == smscnf.COM_POWER:
-        RevealThat('battery', GetBatteryChargeInfo())
+        PerformStatusReport()
 
     elif Command == smscnf.COM_LOCATION:
         behaviourChanged = False
@@ -889,6 +901,78 @@ def PerformDeviceUnlock():
 #   system
 #############################################
 
+def PerformStatusReport():
+    """
+    Gather and reveal all cheap phone status information.
+    """
+    # Assemble short battery status text 
+
+    (BatteryState, chargePercent, isCharging) = GetBatteryChargeInfo(TextOnly = False)
+    BatteryState = '%s%%' % IF(chargePercent == None, '?', chargePercent)
+    if isCharging != None:
+        BatteryState = BatteryState + IF(isCharging, '++', '--')
+
+    # Query device lock mode
+
+    Output = os.popen('dbus-send --system --type=method_call --print-reply \
+                    --dest="com.nokia.mce" "/com/nokia/mce/request" \
+                    com.nokia.mce.request.get_devicelock_mode').read()
+    E = re.compile('string "(\S+)"')
+    try:
+        DeviceLockState = E.findall(Output)[0]
+    except Exception, e:
+        DeviceLockState = 'unknown'
+        LOGGER.warning('Could not determine device lock state: %s' % e)
+
+    # Query screen & keyboard lock mode
+
+    Output = os.popen('dbus-send --system --type=method_call --print-reply \
+                    --dest="com.nokia.mce" "/com/nokia/mce/request" \
+                    com.nokia.mce.request.get_tklock_mode').read()
+    E = re.compile('string "(\S+)"')
+    try:
+        ScreenLockState = E.findall(Output)[0]
+    except Exception, e:
+        ScreenLockState = 'unknown'
+        LOGGER.warning('Could not determine screen lock state: %s' % e)
+
+    # Query device orientation (with interpretation)
+
+    Output = os.popen('dbus-send --system --print-reply \
+                    --dest=com.nokia.mce /com/nokia/mce/request \
+                    com.nokia.mce.request.get_device_orientation').read()
+    E = re.compile('(?:string|int32)\s+"?(\S+?)"?\s+')
+    try:
+        DeviceOrientation = E.findall(Output)
+    except Exception, e:
+        DeviceOrientation = ['unknown', 'unknown', 'unknown', '?', '?', '?']
+        LOGGER.warning('Could not determine current locking state: %s' % e)
+
+    # Assemble status report as short as possible so that it may fit in one SMS
+
+    StatusReport = [
+        'Pwr:%s' % BatteryState,
+        'Kbrd:%s' % GetKeyboardSliderState(),
+        '/'.join([v for v in DeviceOrientation[0:3] if v != 'unknown']),    # interpreted orientation
+#        'Roll:%s Nick:%s Rota:%s' % (DeviceOrientation[3], DeviceOrientation[4], DeviceOrientation[5]),
+        'Light:%s' %  GetAmbientLightValue(),
+        'Proxmty:%s' % GetProximityState(),
+        'Screen:%s' % ScreenLockState,
+        'Device:%s' % DeviceLockState
+        ]
+
+    RevealThat('status', '\n'.join(StatusReport))
+
+def QueryDbus(DbusQueryCommand):
+    try:
+        Output = os.popen(DbusQueryCommand).read()
+        E = re.compile('string "(.*)"')
+        R = E.findall(Output)[0]
+    except:
+        LOGGER.error('Could not get telecom network operator name.')
+        return 'UNKNOWN'        
+
+
 def PerformReboot():
     """
     Reboot the phone.
@@ -1283,6 +1367,7 @@ def GetBatteryChargeInfo(TextOnly = True, ChargePercentMin = 10):
     # Try getting charge info, not all devices support all properties: 
     try:
         chargePercent = dev_obj.GetProperty('battery.charge_level.percentage')  # charge amount of battery in percentage
+        chargePercent = int(float(chargePercent))
         chargePercentText = '%s%%' % chargePercent
         chargeHint = IF(chargePercent >= ChargePercentMin, '', ' (almost empty!)')
     except Exception, e:
@@ -1392,6 +1477,8 @@ def GPSstart(GpsData):
         LOGGER.info('Starting GPS device in location mode.')
     elif GpsMode == 'multi':
         LOGGER.info('Starting GPS device in tracking mode.')
+    else:
+        LOGGER.warning('Starting GPS device in %s mode.' % str(GpsMode))
 
     global GpsStartTime
     GpsStartTime = time.time()
@@ -1425,7 +1512,7 @@ def OnGPSchanged(GpsDevice, GpsData):
         return
 
     TimePassedSeconds = time.time() - GpsStartTime
-    LOGGER.debug('GSP GpsStartTime = %s.' % GpsStartTime)
+    LOGGER.debug('GPS GpsStartTime = %s.' % GpsStartTime)
     LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # LOCATION_GPS_DEVICE_STATUS_NO_FIX  or  LOCATION_GPS_DEVICE_STATUS_FIX
 
     GpsDataOK = True
@@ -1438,13 +1525,19 @@ def OnGPSchanged(GpsDevice, GpsData):
         # GPS device has fix.
         Latitude  = GpsDevice.fix[4]
         Longitude = GpsDevice.fix[5]
-        Accuracy  = GpsDevice.fix[6] / 100
-    
+        Accuracy  = GpsDevice.fix[6]
+
+        # If just the accuracy is unknown then set it to a very high value (better than no coords et all)   
+        if GpsMode == 'multi' and str(Accuracy) == 'nan': 
+            Accuracy = 999999 * 100  # 999999 meters in centimeters. 
+
         # Check if coordinates acquired by GPS device are valid
         GpsDataOK = str(Latitude) != 'nan' and str(Longitude) != 'nan' and str(Accuracy) != 'nan'
         if not GpsDataOK:
             LOGGER.info('Waiting for valid GPS coordinate (search time = %.1f sec.).' % TimePassedSeconds)
 
+        Accuracy  = Accuracy / 100  # centimeters to meters
+
     if not GpsDataOK:
         # check GPS device timout
         if TimePassedSeconds >= int(smscnf.GPSTIMEOUT):
@@ -1777,22 +1870,51 @@ def RestartDaemon():
     finally:
         LOGGER.critical('(RESTART) Not restarted. Failed to kill %s.' % DaemonFile)
 
+def GetAcceleratorValues():
+    """
+    Get current accelerator values.
+    @return string. 3 signed integers X Y Z in range 0..1024 (approx) or ? ? ?
+    """
+    return SaveReadFileLine('/sys/class/i2c-adapter/i2c-3/3-001d/coord',
+                ErrorMessage = 'Failed to read accelerator sensor',
+                DefaultValue = '? ? ?')
+
+def GetProximityState():
+    """
+    Get current ambient light value.
+    @return string. 1 open, closed or unknown
+    """
+    return SaveReadFileLine('/sys/devices/platform/gpio-switch/proximity/state',
+                ErrorMessage = 'Failed to read proximity sensor')
+
+def GetAmbientLightValue():
+    """
+    Get current ambient light value.
+    @return string. 1 integer [lux] or unknown
+    """
+    return SaveReadFileLine('/sys/class/i2c-adapter/i2c-2/2-0029/lux',
+                ErrorMessage = 'Failed to read ambient light sensor')
+
 #############################################
 #   keyboard detect (dbus trigger)
 #############################################
 
+def GetKeyboardSliderState():
+    """
+    Get current keyboard slider state.
+    @return string. 
+    """
+    return SaveReadFileLine('/sys/devices/platform/gpio-switch/slide/state',
+                ErrorMessage = 'Failed to detect keyboard slider state')
+
 def OnKeyboardSliderChange(Action, Type):
     """
     Check and reveal state of keyboard slider.
     @return void. 
     """
-    try:
-        SliderState = open('/sys/devices/platform/gpio-switch/slide/state').read().strip()
-    except Exception, e:
-        LOGGER.error('Failed to detect keyboard slider state: %s' % e)
-        SliderState = 'in unknown state'
-    LOGGER.info('Phone is being used: keyboard now %s.' % SliderState)
-    RevealThat('Keyboard', 'Phone is being used: keyboard now %s.' % SliderState)            
+    SliderState = GetKeyboardSliderState()
+    LOGGER.info('Phone is being used: keyboard position now %s.' % SliderState)
+    RevealThat('Keyboard', 'Phone is being used: keyboard position now %s.' % SliderState)            
 
 #############################################
 #   battery charge (dbus trigger)
@@ -1933,6 +2055,8 @@ def InstallSignalReceivers():
                                     signal_name    = 'battery_state_changed')
     LOGGER.info('Battery status change detection is %s.' % IF(IsBatteryStatusCheckActive, 'on', 'off'))
 
+    # Set CHECKHOST polling
+
     def CheckHost(checktime):
         global IsCheckHostActive
         if smscnf.ENABLECHECKHOST == 'yes':
@@ -1946,8 +2070,31 @@ def InstallSignalReceivers():
             IsCheckHostActive = False
             LOGGER.info('(CHECKHOST) Checkhost is off.')
         return False
+
     if not IsCheckHostActive:
         CheckHost(None)
+    else:
+        LOGGER.info('Checkhost change detection is %s.' % IF(smscnf.ENABLECHECKHOST == 'yes', 'on', 'off'))
+
+def ApplyConfiguration():
+    """
+    Ensure settings of globally loaded smscon configuration are used in device dependend modules.
+    @return void
+    """
+    global smscnf, GpsControl
+    
+    # Set GPS method & most suitable interval
+
+    gps_interval = int(smscnf.GPSINTERVAL)
+    if   gps_interval <= 10:  gps_interval = 10;  pref_interval = location.INTERVAL_10S
+    elif gps_interval <= 20:  gps_interval = 20;  pref_interval = location.INTERVAL_20S
+    elif gps_interval <= 30:  gps_interval = 30;  pref_interval = location.INTERVAL_30S
+    elif gps_interval <= 60:  gps_interval = 60;  pref_interval = location.INTERVAL_60S
+    else:                     gps_interval = 120; pref_interval = location.INTERVAL_120S
+    GpsControl.set_properties(#preferred_method   = location.METHOD_GNSS|location.METHOD_AGNSS,
+                              preferred_method   = location.METHOD_USER_SELECTED,
+                              preferred_interval = pref_interval)
+    LOGGER.info('Preferred GPS query interval is %d sec.' % gps_interval)
 
 def LoadConfiguration():
     """"
@@ -1983,6 +2130,7 @@ def LoadConfiguration():
 
     smscnf = theConf
     if isReload:
+        ApplyConfiguration()
         InstallSignalReceivers()
     return True
 
@@ -2054,24 +2202,17 @@ def main():
         dev_obj = bus.get_object('org.freedesktop.Hal', uids[0])
         LogBatteryChargeInfo()
     
-        # set GPS device control
+        # set global GPS device control
         GpsControl = location.GPSDControl.get_default()
         GpsDevice  = location.GPSDevice()
-    
-        # set GPS method & most suitable interval
-        gps_interval = int(smscnf.GPSINTERVAL)
-        if   gps_interval <= 10:  gps_interval = location.INTERVAL_10S
-        elif gps_interval <= 20:  gps_interval = location.INTERVAL_20S
-        elif gps_interval <= 30:  gps_interval = location.INTERVAL_30S
-        elif gps_interval <= 60:  gps_interval = location.INTERVAL_60S
-        else:                     gps_interval = location.INTERVAL_120S
-        GpsControl.set_properties(preferred_method   = location.METHOD_GNSS|location.METHOD_AGNSS,
-                                  preferred_interval = gps_interval)
-    
+
+        # set GPS signal receivers (need them for DoCommandLine())
         GpsControl.connect('error-verbose', OnGPSerror, gobject.MainLoop() )
         GpsDevice.connect('changed', OnGPSchanged, GpsControl )
         GpsControl.connect('gpsd-stopped', OnGPSstop, gobject.MainLoop() )
-    
+
+        ApplyConfiguration()
+
         CommandLineStatus = DoCommandLine(sys.argv[1:])
         if not CommandLineStatus:
             ExitDaemon('bad command line')
